==> v0.1.zip: collage/src/obj.cpp <==
#include <iostream>
#include <types/model.h>

Vector3 ParseVertexCoordinatesFromOBJ(std::istringstream& stream) {
    float x, y, z;
    stream >> x >> y >> z;
    return {x,y,z};
}

Vector2 ParseVertexTextureCoordinatesFromOBJ(std::istringstream& stream) {
    float u, v;
    stream >> u >> v;
    return {u,v};
}

FaceIndices ParseFaceIndicesFromOBJ(std::istringstream& stream) {
    FaceIndices face{};
    char slash;
    for (int i = 0; i < 3; ++i) {
        unsigned int vI;
        unsigned int tCI;

        stream >> vI >> slash >> tCI;

        face.vertexIndex[i] = vI;
        face.texCoordIndex[i] = tCI;

        // Shift vertexindex by 1
        face.vertexIndex[i]--;
        // shift texcoordindex by 1
        face.texCoordIndex[i]--;
    }
    return face;
}
void Model::loadOBJ(const std::string& filename) {
    std::ifstream file(filename);
    if (!file.is_open())
        std::cerr << filename << " not found." << std::endl;

    std::vector<Vector3> positions;
    std::vector<Vector2> uvs;
    std::string line;

    // First: We read every (unique) vertex entry and textureCoord entry from the file.
    // Then: Use Face data to map the bulk data into correct locations.

    // TODO: Check the assumption that we can **always** read all "v", "vt" etc. tokens  first,
    // allowing us to perform a second-pass to reconstruct faces?

    while (std::getline(file, line)) {
        std::istringstream stream(line);
        std::string prefix;
        stream >> prefix;


        if (prefix == "v") {
            auto result =  ParseVertexCoordinatesFromOBJ(stream);
            positions.emplace_back(result);
        } else if (prefix == "vt") {
            auto result = ParseVertexTextureCoordinatesFromOBJ(stream);
            uvs.emplace_back(result);
        } else if (prefix == "f") {
            auto result = ParseFaceIndicesFromOBJ(stream);
            mapFaces(result, positions, uvs);
        }
    }
    file.close();
};

// map face of model to positions and texture coords
void Model::mapFaces(const FaceIndices& facedata, const std::vector<Vector3>& positions, const std::vector<Vector2>& uvs) {
    TextureInformation tInfo{};
    for (int i = 0; i < 3; ++i) {
        Vector3 vertex;
        Vector2 textureCoordinate;
        vertex = positions[facedata.vertexIndex[i]];
        textureCoordinate = uvs[facedata.texCoordIndex[i]];

        vertices.push_back(vertex);
        tInfo.path = "default"; //Placeholder
        tInfo.textureCoordinates.push_back(textureCoordinate);
        indices.push_back(static_cast<unsigned int>(indices.size()));
    }
    textureInfo.push_back(tInfo);
}

==> v0.2.zip: collage/src/obj.cpp <==
#include <iostream>
#include <types/model.h>

Vector3 ParseVertexCoordinatesFromOBJ(std::istringstream& stream) {
    float x, y, z;
    stream >> x >> y >> z;
    return {x,y,z};
}

Vector2 ParseVertexTextureCoordinatesFromOBJ(std::istringstream& stream) {
    float u, v;
    stream >> u >> v;
    return {u,v};
}

FaceIndices ParseFaceIndicesFromOBJ(std::istringstream& stream) {
    FaceIndices face{};
    char slash;
    for (int i = 0; i < 3; ++i) {
        unsigned int vI;
        unsigned int tCI;

        stream >> vI >> slash >> tCI;

        face.vertexIndex[i] = vI;
        face.texCoordIndex[i] = tCI;

        // Shift vertexindex by 1
        face.vertexIndex[i]--;
        // shift texcoordindex by 1
        face.texCoordIndex[i]--;
    }
    return face;
}
void Model::loadOBJ(const std::string& filename) {
    std::ifstream file(filename);
    if (!file.is_open())
        std::cerr << filename << " not found." << std::endl;

    std::vector<Vector3> positions;
    std::vector<Vector2> uvs;
    std::string line;

    // First: We read every (unique) vertex entry and textureCoord entry from the file.
    // Then: Use Face data to map the bulk data into correct locations.

    // TODO: Check the assumption that we can **always** read all "v", "vt" etc. tokens  first,
    // allowing us to perform a second-pass to reconstruct faces?

    while (std::getline(file, line)) {
        std::istringstream stream(line);
        std::string prefix;
        stream >> prefix;


        if (prefix == "v") {
            auto result =  ParseVertexCoordinatesFromOBJ(stream);
            positions.emplace_back(result);
        } else if (prefix == "vt") {
            auto result = ParseVertexTextureCoordinatesFromOBJ(stream);
            uvs.emplace_back(result);
        } else if (prefix == "f") {
            auto result = ParseFaceIndicesFromOBJ(stream);
            mapFaces(result, positions, uvs);
        }
    }
    file.close();
    textureInfo.push_back({"", uvs});
};

// map face of model to positions and texture coords
void Model::mapFaces(const FaceIndices& facedata, const std::vector<Vector3>& positions, const std::vector<Vector2>& uvs) {
    std::vector<Vector2> textureCoordinates;
    for (int i = 0; i < 3; ++i) {
        Vector3 vertex;
        //Vector2 textureCoordinate;
        vertex = positions[facedata.vertexIndex[i]];
        //textureCoordinate = uvs[facedata.texCoordIndex[i]];
        vertices.push_back(vertex);
        indices.push_back(static_cast<unsigned int>(indices.size()));
        //textureCoordinates.push_back(textureCoordinate);
    }
    textureInfo.push_back({"", textureCoordinates});
}
